var checkBoxes = [];
var kmlLayers = [];
var imgLayers = [];

var markerObjectArray = [];
var map;

var	thumbnailWidth = '300';
var jsonDocUrl = "./records.json";
var	lat = 34.7;
var lng = 135.5;
var	zlv = 2;
 
var jsonMarkers;
var infoWindowContent;
 

var mapTypes = {};
var mapTypeIds = []; 

    mapTypes['sky'] = {
      getTileUrl: function(coord, zoom) {
        return getHorizontallyRepeatingTileUrl(coord, zoom, function(coord, zoom) {
          return "http://mw1.google.com/mw-planetary/sky/skytiles_v1/" +
                 coord.x + "_" + coord.y + '_' + zoom + '.jpg';
 
        });
      },
      tileSize: new google.maps.Size(256, 256),
      isPng: false,
      maxZoom: 13,
      radius: 57.2957763671875,
      name: 'Sky',
      credit: 'Image Credit: SDSS, DSS Consortium, NASA/ESA/STScI'
    };

function getHorizontallyRepeatingTileUrl(coord, zoom, urlfunc) 
{
    var y = coord.y;
    var x = coord.x;
      // tile range in one direction range is dependent on zoom level
      // 0 = 1 tile, 1 = 2 tiles, 2 = 4 tiles, 3 = 8 tiles, etc
    var tileRange = 1 << zoom;
      // don't repeat across y-axis (vertically)
    if (y < 0 || y >= tileRange) { return null; }
 
      // repeat across x-axis
    if (x < 0 || x >= tileRange) {
        x = (x % tileRange + tileRange) % tileRange;
      }
 
    return urlfunc({x:x,y:y}, zoom)
}

//Controls
function setCredit(credit) 
{
	creditNode.innerHTML = credit + ' ';
}
 
function reload()
{
	deleteAllLayers();
	deleteMarkers();
	jsonMarkers.marker.forEach(addMarkers);
	
	resetZoom();
}

function createTemporaryMarker(coord)
{	var tmpMarkerObject = new Object();
	tmpMarkerObject.marker = new google.maps.Marker(	
	{	position: coord, map: map, title: 'Temp. Marker', draggable: true, label: 'T',
		animation: google.maps.Animation.DROP
	});
    var	strLat = coord.lat().toFixed(6).toString();
    var	strLng = coord.lng().toFixed(6).toString();

	var contentString =  '<a href="yoururl.com?coord='+strLat+';'+strLng+';15'+'">'
						+'<font class="infowindow">'+  '&quot;lat&quot;: &quot;'+strLat+'&quot;,' + '</font><br>'
						+'<font class="infowindow">'+  '&quot;lng&quot;: &quot;'+strLng+'&quot;,' + '</font></a>';
	tmpMarkerObject.infoWindow = new google.maps.InfoWindow({content: contentString, maxWidth: '300'});
	
	markerObjectArray.push(tmpMarkerObject);	

	registerMarkerEventListener(tmpMarkerObject);
}

function initialize() 
{	for (var key in mapTypes) {mapTypeIds.push(key);}
 	
 	var mapOptions = 
	{	zoom: zlv, minZoom: 2, center: new google.maps.LatLng(lat, lng),
		panControl: false, zoomControl: false, scaleControl: true, streetViewControl: false, fullscreenControl: false,
		mapTypeControl: false
	};
	
	map = new google.maps.Map(document.getElementById('map-canvas'), mapOptions);
	
	map.controls[google.maps.ControlPosition.BOTTOM_RIGHT].push(creditNode);
 
    for (key in mapTypes) {
		map.mapTypes.set(key, new google.maps.ImageMapType(mapTypes[key]));
    }
    setCredit("Image Credit: SDSS, DSS Consortium, NASA/ESA/STScI");
    map.setMapTypeId('sky');
     
	google.maps.event.addListener( map, 'click', 
		 function(e){	if(window.event.ctrlKey || window.event.metaKey)	
						 createTemporaryMarker(e.latLng);
					 }	); 

	
	jsonMarkers = JSON.parse(httpGet(jsonDocUrl));

	deleteAllLayers();
 	deleteMarkers();
	
	jsonMarkers.marker.forEach(addMarkers);
}

function deleteMarkers() 
{	for (var i = 0; i < markerObjectArray.length; i++) 
	{	markerObjectArray[i].marker.setMap(null);
		if(markerObjectArray[i].infoWindow != 0) markerObjectArray[i].infoWindow.close();
	}
 	 markerObjectArray = [];
}

function deleteAllLayers() 
{	for (var i = 0; i < kmlLayers.length; i++) kmlLayers[i].setMap(null);
  	for (var i = 0; i < checkBoxes.length; i++) checkBoxes[i].checked = false;
	for (var i = 0; i < imgLayers.length; i++) imgLayers[i].setMap(null);
	kmlLayers = [];
	checkBoxes = [];
	imgLayers = [];
}

function checkBoxControl(checkBoxName, checkState)
{	for (var i = 0; i < checkBoxes.length; i++) 
		if(checkBoxes[i].name == checkBoxName) checkBoxes[i].checked = checkState;
}

function deleteLayers(kmlFileName, checkBoxName) 
{	var tmpLayers =[];
	for (var i = 0; i < kmlLayers.length; i++) 
	{	if(kmlLayers[i].getUrl() == kmlFileName) kmlLayers[i].setMap(null);
		else tmpLayers.push(kmlLayers[i]);
	}
	for (var i = 0; i < checkBoxes.length; i++) 
		if(checkBoxes[i].name == checkBoxName) checkBoxes[i].checked = false;
	
	kmlLayers = [];
	kmlLayers = tmpLayers;
	
	if(kmlLayers.length == 0&&imgLayers.length ==0) map.setZoom(zlv);
	else if(imgLayers.length ==0)
		map.fitBounds(kmlLayers[kmlLayers.length - 1].getDefaultViewport());
	else if(klmLayers.length ==0)
		map.fitBounds(imgLayers[imgLayers.length - 1].getDefaultViewport());
}

function resetZoom() 
{	map.setZoom(zlv);
	if(isMobile) 
	{	var myLatlng = new google.maps.LatLng(15.0, 127.378857);  
		map.setCenter(myLatlng);
}	}

function addMarkers(thisMarker,index)
{	
	var title = thisMarker.id;
	infoWindowContent = thisMarker.desc;
		
	if(thisMarker.overLayer)
	{
		if(thisMarker.overLayer.length >1)
			thisMarker.overLayer.forEach(addOverlayer); 
		else
			addOverlayer(thisMarker.overLayer);
	}
	if(thisMarker.mapLayer)
		addOverlayer(thisMarker.mapLayer);
				
	infoWindowContent += '<a href="'+thisMarker.url+'" title="Click to redirect to Flicker!" target="_blank"><br>' +
					 '<img src="' + thisMarker.thumbnail + 
					 '"' +thisMarker.portrait+'=' + thumbnailWidth + '></a>';
	
	var markerObject = new Object();
	markerObject.lat = thisMarker.lat;
	markerObject.lng = thisMarker.lng;
	markerObject.title = thisMarker.id;
	markerObject.content = infoWindowContent;
	markerObject.thumbnailWidth = thumbnailWidth;
	markerObject.infoWindow = 0;
	//markerObject.infoWindow = new google.maps.InfoWindow({content: infoWindowContent, maxWidth: thumbnailWidth});
	markerObject.location = thisMarker.group;
	markerObject.camera = thisMarker.camera;
	
	renderMarkers(markerObject,(index/2)*50);
}

function addOverlayer(thisLayer)
{
	var funcName;
	if(thisLayer.type.toLowerCase() == "map") 
		funcName = "addMapOverLayer(this);";
	else funcName = "addRouteOverLayer(this);";
				
	var strCheck = "";
				
	infoWindowContent += '<br><input onclick="'+funcName+'" type=checkbox name="'+
						 thisLayer.id +'" value="' + thisLayer.data +'"'+ strCheck+'>';
	if(thisLayer.type.toLowerCase() != "map") 				 
	{	
		infoWindowContent += '<font class="checkbox">' + thisLayer.label+ '</font>'+'</a>';	
	}
	else
	{
		infoWindowContent += '<font class="checkbox">' + thisLayer.label+ '</font>';	
	}
}

function renderMarkers(markerObject, timeout)
{		window.setTimeout(function()
	{	markerObject.marker = new google.maps.Marker(	
		{	position: new google.maps.LatLng(markerObject.lat, markerObject.lng), 
			map: map, title: markerObject.title, animation: google.maps.Animation.DROP
		});
		
		markerObjectArray.push(markerObject);
		registerMarkerEventListener(markerObject);
	}, timeout);
}

//Adding Markers and Layers by Google maps API
function registerMarkerEventListener(markerObject)
{	google.maps.event.addListener(markerObject.marker, 'click', function() 
	{	if(	markerObject.marker.getAnimation() == null)
		{	
			if(markerObject.infoWindow == 0)
			{
				markerObject.infoWindow = new google.maps.InfoWindow({content: markerObject.content, maxWidth: markerObject.thumbnailWidth});	
				google.maps.event.addListener( markerObject.infoWindow, 'closeclick', function(){markerObject.marker.setAnimation(null);});
			}
			if(markerObject.infoWindow != 0) markerObject.infoWindow.open(map, markerObject.marker);
			markerObject.marker.setAnimation(google.maps.Animation.BOUNCE); 
		}
		else
		{	markerObject.marker.setAnimation(null); 
			if(markerObject.infoWindow != 0)markerObject.infoWindow.close();
		}
		for(var i=0; i<markerObjectArray.length;i++)
		{	if(markerObjectArray[i] == markerObject) continue;
			if(!event.ctrlKey && !event.metaKey)
			{	if(markerObject.infoWindow != 0)markerObjectArray[i].infoWindow.close();
				markerObjectArray[i].marker.setAnimation(null);
			}
		}			
	}	);
}

function addMapOverLayer(checkBox)
{	if(checkBox.checked) 
	{	for (var i=0; i<jsonMarkers.marker.length; i++)
		{	if(jsonMarkers.marker[i].mapLayer)
			{	var layer = jsonMarkers.marker[i].mapLayer;
				if(checkBox.name == layer.id)
				{	
					var imgBounds = new google.maps.LatLngBounds(
						new google.maps.LatLng(layer.lat1, layer.lng1) ,
						new google.maps.LatLng(layer.lat2, layer.lng2) );
		
					var maplayer = new google.maps.GroundOverlay(
					strUrl + layer.data, imgBounds);
				
					maplayer.setMap(map); map.fitBounds(imgBounds);
					imgLayers.push(maplayer);		
				}
			}
		}
		checkBoxes.push(checkBox);
	}
	else
	{	for(var k=0; k<imgLayers.length; k++){ imgLayers[k].setMap(null);}
		imgLayers = []; 
	}	
}

function addRouteOverLayer(checkBox)
{	if(checkBox.checked) 
	{	var Layer = new google.maps.KmlLayer({url: checkBox.value});
		Layer.setMap(map); 
		kmlLayers.push(Layer);
		checkBoxes.push(checkBox);
	}
	else deleteLayers(checkBox.value, checkBox.name);
}

function httpGet(theUrl)
{
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open( "GET", theUrl, false ); // false for synchronous request
    xmlHttp.send( null );
    return xmlHttp.responseText;
}